@@ -3,19 +3,16 @@ use bevy_ecs_tilemap::prelude::*;
 
 mod helpers;
 
+#[derive(Component)]
 struct CurrentColor(u16);
+
+#[derive(Component)]
 struct LastUpdate(f64);
 
-fn startup(
-    mut commands: Commands,
-    asset_server: Res<AssetServer>,
-    mut materials: ResMut<Assets<ColorMaterial>>,
-    mut map_query: MapQuery,
-) {
+fn startup(mut commands: Commands, asset_server: Res<AssetServer>, mut map_query: MapQuery) {
     commands.spawn_bundle(OrthographicCameraBundle::new_2d());
 
     let texture_handle = asset_server.load("tiles.png");
-    let material_handle = materials.add(ColorMaterial::texture(texture_handle));
 
     // We can create maps by using the LayerBuilder
     // LayerBuilder creates the tile entities and makes sure they are attached to chunks correctly.
@@ -29,10 +26,10 @@ fn startup(
     let (mut layer_builder, layer_entity) = LayerBuilder::<TileBundle>::new(
         &mut commands,
         LayerSettings::new(
-            UVec2::new(4, 4),
-            UVec2::new(32, 32),
-            Vec2::new(16.0, 16.0),
-            Vec2::new(96.0, 256.0),
+            MapSize(4, 4),
+            ChunkSize(32, 32),
+            TileSize(16.0, 16.0),
+            TextureSize(96.0, 16.0),
         ),
         0u16,
         0u16,
@@ -43,8 +40,8 @@ fn startup(
 
     // You can also fill in a portion of the map
     layer_builder.fill(
-        UVec2::new(0, 0),
-        UVec2::new(10, 10),
+        TilePos(0, 0),
+        TilePos(10, 10),
         Tile {
             texture_index: 1,
             ..Default::default()
@@ -52,27 +49,24 @@ fn startup(
         .into(),
     );
 
-    let neighbors = layer_builder.get_tile_neighbors(UVec2::new(0, 0));
+    let neighbors = layer_builder.get_tile_neighbors(TilePos(0, 0));
 
     // We can access tiles like normal using:
-    assert!(layer_builder.get_tile(UVec2::new(0, 0)).is_ok());
+    assert!(layer_builder.get_tile(TilePos(0, 0)).is_ok());
     assert!(neighbors.len() == 8);
-    assert!(neighbors.iter().filter(|n| n.1.is_some()).count() == 3); // Only 3 neighbors since negative is outside of map.
+    let neighbor_count = neighbors.iter().filter(|n| n.is_some()).count();
+    assert!(neighbor_count == 3); // Only 3 neighbors since negative is outside of map.
 
     let mut color = 0;
     for x in (2..128).step_by(4) {
         color += 1;
         for y in (2..128).step_by(4) {
             // Grabbing neighbors is easy.
-            let neighbors: Vec<IVec2> = layer_builder
-                .get_tile_neighbors(UVec2::new(x, y))
-                .iter()
-                .map(|(pos, _)| *pos)
-                .collect();
-            for pos in neighbors.iter() {
+            let neighbors = get_neighboring_pos(TilePos(x, y));
+            for &pos in neighbors.iter() {
                 // We can set specific tiles like this:
                 let _ = layer_builder.set_tile(
-                    UVec2::new(pos.x as u32, pos.y as u32),
+                    pos.expect("Tile position does not exist."),
                     Tile {
                         texture_index: color,
                         ..Default::default()
@@ -84,7 +78,7 @@ fn startup(
     }
 
     // Once create_layer is called you can no longer access the tiles in this system.
-    map_query.build_layer(&mut commands, layer_builder, material_handle);
+    map_query.build_layer(&mut commands, layer_builder, texture_handle);
 
     commands
         .entity(layer_entity)
@@ -126,20 +120,29 @@ fn update_map(
             for x in (2..128).step_by(4) {
                 for y in (2..128).step_by(4) {
                     // First we get the neighboring entities for the given tile.
-                    let neighbors = map_query.get_tile_neighbors(UVec2::new(x, y), 0u16, 0u16);
-                    for (pos, neighbor) in neighbors.iter() {
-                        // If the tile exists we will have an entity.
-                        if let Some(neighbor) = neighbor {
+                    let neighboring_tile_pos = get_neighboring_pos(TilePos(x, y));
+                    let neighboring_entities =
+                        map_query.get_tile_neighbors(TilePos(x, y), 0u16, 0u16);
+
+                    // Iterating over each neighbor
+                    for i in 0..8 {
+                        if neighboring_entities[i].is_ok() {
                             // We query tiles using a query coming from this system.
                             // This has the add advantage of being able to query "extra" data per tile.
-                            if let Ok(mut tile) = tile_query.get_mut(*neighbor) {
+                            if let Ok(mut tile) =
+                                tile_query.get_mut(neighboring_entities[i].unwrap())
+                            {
                                 *tile = Tile {
                                     texture_index: color,
                                     ..Default::default()
                                 };
                                 // Finally after mutating the tile we can tell the internal systems to "remesh" the tilemap.
                                 // This sends the new tile data to the gpu.
-                                map_query.notify_chunk_for_tile(pos.as_u32(), 0u16, 0u16);
+                                map_query.notify_chunk_for_tile(
+                                    neighboring_tile_pos[i].unwrap(),
+                                    0u16,
+                                    0u16,
+                                );
                             }
                         }
                     }
@@ -156,11 +159,7 @@ fn update_map(
 }
 
 fn main() {
-    env_logger::Builder::from_default_env()
-        .filter_level(log::LevelFilter::Info)
-        .init();
-
-    App::build()
+    App::new()
         .insert_resource(WindowDescriptor {
             width: 1270.0,
             height: 720.0,
@@ -169,9 +168,9 @@ fn main() {
         })
         .add_plugins(DefaultPlugins)
         .add_plugin(TilemapPlugin)
-        .add_startup_system(startup.system())
-        .add_system(update_map.system())
-        .add_system(helpers::camera::movement.system())
-        .add_system(helpers::texture::set_texture_filters_to_nearest.system())
+        .add_startup_system(startup)
+        .add_system(update_map)
+        .add_system(helpers::camera::movement)
+        .add_system(helpers::texture::set_texture_filters_to_nearest)
         .run();
 }
