@@ -3,16 +3,10 @@ use bevy_ecs_tilemap::prelude::*;
 
 mod helpers;
 
-fn startup(
-    mut commands: Commands,
-    asset_server: Res<AssetServer>,
-    mut materials: ResMut<Assets<ColorMaterial>>,
-    mut map_query: MapQuery,
-) {
+fn startup(mut commands: Commands, asset_server: Res<AssetServer>, mut map_query: MapQuery) {
     commands.spawn_bundle(OrthographicCameraBundle::new_2d());
 
     let texture_handle = asset_server.load("tiles.png");
-    let material_handle = materials.add(ColorMaterial::texture(texture_handle));
 
     // Create map entity and component:
     let map_entity = commands.spawn().id();
@@ -22,10 +16,10 @@ fn startup(
     let (mut layer_builder, _) = LayerBuilder::new(
         &mut commands,
         LayerSettings::new(
-            UVec2::new(2, 2),
-            UVec2::new(8, 8),
-            Vec2::new(16.0, 16.0),
-            Vec2::new(96.0, 256.0),
+            MapSize(2, 2),
+            ChunkSize(8, 8),
+            TileSize(16.0, 16.0),
+            TextureSize(96.0, 16.0),
         ),
         0u16,
         0u16,
@@ -35,7 +29,7 @@ fn startup(
 
     // Builds the layer.
     // Note: Once this is called you can no longer edit the layer until a hard sync in bevy.
-    let layer_entity = map_query.build_layer(&mut commands, layer_builder, material_handle);
+    let layer_entity = map_query.build_layer(&mut commands, layer_builder, texture_handle);
 
     // Required to keep track of layers for a map internally.
     map.add_layer(&mut commands, 0u16, layer_entity);
@@ -50,11 +44,11 @@ fn startup(
 }
 
 fn main() {
-    env_logger::Builder::from_default_env()
-        .filter_level(log::LevelFilter::Info)
-        .init();
+    // env_logger::Builder::from_default_env()
+    //     .filter_level(log::LevelFilter::Info)
+    //     .init();
 
-    App::build()
+    App::new()
         .insert_resource(WindowDescriptor {
             width: 1270.0,
             height: 720.0,
@@ -63,8 +57,8 @@ fn main() {
         })
         .add_plugins(DefaultPlugins)
         .add_plugin(TilemapPlugin)
-        .add_startup_system(startup.system())
-        .add_system(helpers::camera::movement.system())
-        .add_system(helpers::texture::set_texture_filters_to_nearest.system())
+        .add_startup_system(startup)
+        .add_system(helpers::camera::movement)
+        .add_system(helpers::texture::set_texture_filters_to_nearest)
         .run();
 }
