@@ -1,34 +1,31 @@
 use std::collections::HashSet;
 
 use bevy::{
+    core::Time,
     diagnostic::{FrameTimeDiagnosticsPlugin, LogDiagnosticsPlugin},
     prelude::*,
+    render::camera::OrthographicCameraBundle,
+    window::WindowDescriptor,
 };
 use bevy_ecs_tilemap::prelude::*;
 use rand::{thread_rng, Rng};
 
 mod helpers;
 
-fn startup(
-    mut commands: Commands,
-    asset_server: Res<AssetServer>,
-    mut materials: ResMut<Assets<ColorMaterial>>,
-    mut map_query: MapQuery,
-) {
+fn startup(mut commands: Commands, asset_server: Res<AssetServer>, mut map_query: MapQuery) {
     commands.spawn_bundle(OrthographicCameraBundle::new_2d());
 
     let texture_handle = asset_server.load("tiles.png");
-    let material_handle = materials.add(ColorMaterial::texture(texture_handle));
 
     // Create map entity and component:
     let map_entity = commands.spawn().id();
     let mut map = Map::new(0u16, map_entity);
 
     let layer_settings = LayerSettings::new(
-        UVec2::new(10, 10),
-        UVec2::new(64, 64),
-        Vec2::new(16.0, 16.0),
-        Vec2::new(96.0, 256.0),
+        MapSize(10, 10),
+        ChunkSize(64, 64),
+        TileSize(16.0, 16.0),
+        TextureSize(96.0, 16.0),
     );
 
     let center = layer_settings.get_pixel_center();
@@ -41,11 +38,16 @@ fn startup(
     layer_builder.for_each_tiles_mut(|tile_entity, tile_data| {
         // True here refers to tile visibility.
         *tile_data = Some(TileBundle::default());
-        // Be careful here as this entity can sometimes not have any tile data.
-        commands.entity(tile_entity).insert(LastUpdate::default());
+        // Tile entity might not exist at this point so you'll need to create it.
+        if tile_entity.is_none() {
+            *tile_entity = Some(commands.spawn().id());
+        }
+        commands
+            .entity(tile_entity.unwrap())
+            .insert(LastUpdate::default());
     });
 
-    map_query.build_layer(&mut commands, layer_builder, material_handle);
+    map_query.build_layer(&mut commands, layer_builder, texture_handle);
 
     // Spawn Map
     // Required in order to use map_query to retrieve layers/tiles.
@@ -56,7 +58,7 @@ fn startup(
         .insert(GlobalTransform::default());
 }
 
-#[derive(Default)]
+#[derive(Default, Component)]
 struct LastUpdate {
     value: f64,
 }
@@ -87,11 +89,7 @@ fn random(
 }
 
 fn main() {
-    env_logger::Builder::from_default_env()
-        .filter_level(log::LevelFilter::Warn)
-        .init();
-
-    App::build()
+    App::new()
         .insert_resource(WindowDescriptor {
             width: 1270.0,
             height: 720.0,
@@ -102,9 +100,9 @@ fn main() {
         .add_plugin(LogDiagnosticsPlugin::default())
         .add_plugin(FrameTimeDiagnosticsPlugin::default())
         .add_plugin(TilemapPlugin)
-        .add_startup_system(startup.system())
-        .add_system(random.system())
-        .add_system(helpers::camera::movement.system())
-        .add_system(helpers::texture::set_texture_filters_to_nearest.system())
+        .add_startup_system(startup)
+        .add_system(random)
+        .add_system(helpers::camera::movement)
+        .add_system(helpers::texture::set_texture_filters_to_nearest)
         .run();
 }
